@@ -6,6 +6,6 @@ SCRIPT_DIR=$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )
 
 cd "$SCRIPT_DIR"
 
-bash build.sh
+python build.py
 
 rsync --delete --archive --stats --progress --human-readable book/html/ shark.fish:numbat.dev/doc/
