@@ -10,14 +10,14 @@ use config::{ColorMode, Config, ExchangeRateFetchingPolicy, IntroBanner, PrettyP
 use highlighter::NumbatHighlighter;
 
 use itertools::Itertools;
-use numbat::command::{self, CommandParser, SourcelessCommandParser};
+use numbat::command::{CommandControlFlow, CommandRunner};
+use numbat::compact_str::CompactString;
 use numbat::diagnostic::ErrorDiagnostic;
-use numbat::help::help_markup;
 use numbat::markup as m;
 use numbat::module_importer::{BuiltinModuleImporter, ChainedImporter, FileSystemImporter};
 use numbat::pretty_print::PrettyPrint;
 use numbat::resolver::CodeSource;
-use numbat::session_history::{ParseEvaluationResult, SessionHistory, SessionHistoryOptions};
+use numbat::session_history::{ParseEvaluationResult, SessionHistory};
 use numbat::{Context, NumbatError};
 use numbat::{InterpreterSettings, NameResolutionError};
 
@@ -132,6 +132,26 @@ struct Cli {
 }
 
 impl Cli {
+    fn make_fresh_context() -> Context {
+        let mut fs_importer = FileSystemImporter::default();
+        for path in Self::get_modules_paths() {
+            fs_importer.add_path(path);
+        }
+
+        let importer = ChainedImporter::new(
+            Box::new(fs_importer),
+            Box::<BuiltinModuleImporter>::default(),
+        );
+
+        let mut context = Context::new(importer);
+
+        context.set_terminal_width(
+            terminal_size::terminal_size().map(|(terminal_size::Width(w), _)| w as usize),
+        );
+
+        context
+    }
+
     fn new(args: Args) -> Result<Self> {
         let user_config_path = Self::get_config_path().join("config.toml");
 
@@ -156,23 +176,9 @@ impl Cli {
         config.enter_repl =
             (args.file.is_none() && args.expression.is_none()) || args.inspect_interactively;
 
-        let mut fs_importer = FileSystemImporter::default();
-        for path in Self::get_modules_paths() {
-            fs_importer.add_path(path);
-        }
-
-        let importer = ChainedImporter::new(
-            Box::new(fs_importer),
-            Box::<BuiltinModuleImporter>::default(),
-        );
-
-        let mut context = Context::new(importer);
+        let mut context = Self::make_fresh_context();
         context.set_debug(args.debug);
 
-        context.set_terminal_width(
-            terminal_size::terminal_size().map(|(terminal_size::Width(w), _)| w as usize),
-        );
-
         Ok(Self {
             context: Arc::new(Mutex::new(context)),
             config,
@@ -182,6 +188,10 @@ impl Cli {
     }
 
     fn run(&mut self) -> Result<()> {
+        // Enabled ANSI colors on Windows 10
+        #[cfg(windows)]
+        colored::control::set_virtual_terminal(true).unwrap();
+
         match self.config.color {
             ColorMode::Never => SHOULD_COLORIZE.set_override(false),
             ColorMode::Always => SHOULD_COLORIZE.set_override(true),
@@ -297,7 +307,10 @@ impl Cli {
             completer: NumbatCompleter {
                 context: self.context.clone(),
                 modules: self.context.lock().unwrap().list_modules().collect(),
-                all_timezones: jiff::tz::db().available().collect(),
+                all_timezones: jiff::tz::db()
+                    .available()
+                    .map(CompactString::from)
+                    .collect(),
             },
             highlighter: NumbatHighlighter {
                 context: self.context.clone(),
@@ -347,123 +360,65 @@ impl Cli {
         rl: &mut Editor<NumbatHelper, DefaultHistory>,
         interactive: bool,
     ) -> Result<()> {
-        let mut session_history = SessionHistory::default();
+        let mut cmd_runner = CommandRunner::<Editor<NumbatHelper, DefaultHistory>>::new()
+            .print_with(|m| println!("{}", ansi_format(m, true)))
+            .enable_clear(|rl| match rl.clear_screen() {
+                Ok(_) => CommandControlFlow::Continue,
+                Err(_) => CommandControlFlow::Return,
+            })
+            .enable_save(SessionHistory::default())
+            .enable_reset(Self::make_fresh_context)
+            .enable_quit();
 
         loop {
             let readline = rl.readline(&self.config.prompt);
             match readline {
                 Ok(line) => {
-                    if !line.trim().is_empty() {
-                        rl.add_history_entry(&line)?;
+                    if line.trim().is_empty() {
+                        continue;
+                    }
 
-                        // if we enter here, the line looks like a command
-                        if let Some(sourceless_parser) = SourcelessCommandParser::new(&line) {
-                            let mut parser = CommandParser::new(
-                                sourceless_parser,
-                                self.context
-                                    .lock()
-                                    .unwrap()
-                                    .resolver_mut()
-                                    .add_code_source(CodeSource::Text, &line),
-                            );
-
-                            match parser.parse_command() {
-                                Ok(command) => match command {
-                                    command::Command::Help => {
-                                        let help = help_markup();
-                                        print!("{}", ansi_format(&help, true));
-                                        // currently, the ansi formatter adds indents
-                                        // _after_ each newline and so we need to manually
-                                        // add an extra blank line to absorb this indent
-                                        println!();
-                                    }
-                                    command::Command::Info { item } => {
-                                        let help = self
-                                            .context
-                                            .lock()
-                                            .unwrap()
-                                            .print_info_for_keyword(item);
-                                        println!("{}", ansi_format(&help, true));
-                                    }
-                                    command::Command::List { items } => {
-                                        let context = self.context.lock().unwrap();
-                                        let m = match items {
-                                            None => context.print_environment(),
-                                            Some(command::ListItems::Functions) => {
-                                                context.print_functions()
-                                            }
-                                            Some(command::ListItems::Dimensions) => {
-                                                context.print_dimensions()
-                                            }
-                                            Some(command::ListItems::Variables) => {
-                                                context.print_variables()
-                                            }
-                                            Some(command::ListItems::Units) => {
-                                                context.print_units()
-                                            }
-                                        };
-                                        println!("{}", ansi_format(&m, false));
-                                    }
-                                    command::Command::Clear => rl.clear_screen()?,
-                                    command::Command::Save { dst } => {
-                                        let save_result = session_history.save(
-                                            dst,
-                                            SessionHistoryOptions {
-                                                include_err_lines: false,
-                                                trim_lines: true,
-                                            },
-                                        );
-                                        match save_result {
-                                            Ok(_) => {
-                                                let m = m::text(
-                                                    "successfully saved session history to",
-                                                ) + m::space()
-                                                    + m::string(dst.to_string());
-                                                println!("{}", ansi_format(&m, interactive));
-                                            }
-                                            Err(err) => {
-                                                self.print_diagnostic(*err);
-                                                continue;
-                                            }
-                                        }
-                                    }
-                                    command::Command::Quit => return Ok(()),
-                                },
-                                Err(e) => {
-                                    self.print_diagnostic(e);
-                                    continue;
-                                }
-                            }
+                    rl.add_history_entry(&line)?;
 
+                    let mut ctx = self.context.lock().unwrap();
+                    match cmd_runner.try_run_command(&line, &mut ctx, rl) {
+                        Ok(cf) => match cf {
+                            CommandControlFlow::Continue => continue,
+                            CommandControlFlow::Return => return Ok(()),
+                            CommandControlFlow::NotACommand => {}
+                        },
+                        Err(err) => {
+                            ctx.print_diagnostic(*err);
                             continue;
                         }
-
-                        let ParseEvaluationOutcome {
-                            control_flow,
-                            result,
-                        } = self.parse_and_evaluate(
-                            &line,
-                            CodeSource::Text,
-                            if interactive {
-                                ExecutionMode::Interactive
-                            } else {
-                                ExecutionMode::Normal
-                            },
-                            self.config.pretty_print,
-                        );
-
-                        match control_flow {
-                            std::ops::ControlFlow::Continue(()) => {}
-                            std::ops::ControlFlow::Break(ExitStatus::Success) => {
-                                return Ok(());
-                            }
-                            std::ops::ControlFlow::Break(ExitStatus::Error) => {
-                                bail!("Interpreter stopped due to error")
-                            }
-                        }
-
-                        session_history.push(line, result);
                     }
+                    drop(ctx);
+
+                    let ParseEvaluationOutcome {
+                        control_flow,
+                        result,
+                    } = self.parse_and_evaluate(
+                        &line,
+                        CodeSource::Text,
+                        if interactive {
+                            ExecutionMode::Interactive
+                        } else {
+                            ExecutionMode::Normal
+                        },
+                        self.config.pretty_print,
+                    );
+
+                    match control_flow {
+                        std::ops::ControlFlow::Continue(()) => {}
+                        std::ops::ControlFlow::Break(ExitStatus::Success) => {
+                            return Ok(());
+                        }
+                        std::ops::ControlFlow::Break(ExitStatus::Error) => {
+                            bail!("Interpreter stopped due to error")
+                        }
+                    }
+
+                    cmd_runner.push_to_history(&line, result);
                 }
                 Err(ReadlineError::Interrupted) => {}
                 Err(ReadlineError::Eof) => {
