@@ -1,5 +1,6 @@
 use std::collections::HashMap;
 
+use compact_str::{CompactString, ToCompactString};
 use itertools::Itertools;
 
 use crate::ast::ProcedureKind;
@@ -23,15 +24,15 @@ use crate::{decorator, ffi, Type};
 
 #[derive(Debug, Clone, Default)]
 pub struct LocalMetadata {
-    pub name: Option<String>,
-    pub url: Option<String>,
-    pub description: Option<String>,
-    pub aliases: Vec<String>,
+    pub name: Option<CompactString>,
+    pub url: Option<CompactString>,
+    pub description: Option<CompactString>,
+    pub aliases: Vec<CompactString>,
 }
 
 #[derive(Debug, Clone)]
 pub struct Local {
-    identifier: String,
+    identifier: CompactString,
     depth: usize,
     pub metadata: LocalMetadata,
 }
@@ -42,9 +43,9 @@ pub struct BytecodeInterpreter {
     /// List of local variables currently in scope, one vector for each scope (for now: 0: 'global' scope, 1: function scope)
     locals: Vec<Vec<Local>>,
     // Maps names of units to indices of the respective constants in the VM
-    unit_name_to_constant_index: HashMap<String, u16>,
+    unit_name_to_constant_index: HashMap<CompactString, u16>,
     /// List of functions
-    functions: HashMap<String, bool>,
+    functions: HashMap<CompactString, bool>,
 }
 
 impl BytecodeInterpreter {
@@ -61,12 +62,12 @@ impl BytecodeInterpreter {
 
                 if let Some(position) = self.locals[current_depth]
                     .iter()
-                    .rposition(|l| &l.identifier == identifier && l.depth == current_depth)
+                    .rposition(|l| l.identifier == identifier && l.depth == current_depth)
                 {
                     self.vm.add_op1(Op::GetLocal, position as u16); // TODO: check overflow
                 } else if let Some(upvalue_position) = self.locals[0]
                     .iter()
-                    .rposition(|l| &l.identifier == identifier)
+                    .rposition(|l| l.identifier == identifier)
                 {
                     self.vm.add_op1(Op::GetUpvalue, upvalue_position as u16);
                 } else if LAST_RESULT_IDENTIFIERS.contains(identifier) {
@@ -75,9 +76,9 @@ impl BytecodeInterpreter {
                     let index = self
                         .vm
                         .add_constant(Constant::FunctionReference(if *is_foreign {
-                            FunctionReference::Foreign(identifier.to_string())
+                            FunctionReference::Foreign(identifier.to_compact_string())
                         } else {
-                            FunctionReference::Normal(identifier.to_string())
+                            FunctionReference::Normal(identifier.to_compact_string())
                         }));
                     self.vm.add_op1(Op::LoadConstant, index);
                 } else {
@@ -221,7 +222,7 @@ impl BytecodeInterpreter {
                 for part in string_parts {
                     match part {
                         StringPart::Fixed(s) => {
-                            let index = self.vm.add_constant(Constant::String(s.to_string()));
+                            let index = self.vm.add_constant(Constant::String(s.clone()));
                             self.vm.add_op1(Op::LoadConstant, index)
                         }
                         StringPart::Interpolation {
@@ -231,7 +232,7 @@ impl BytecodeInterpreter {
                         } => {
                             self.compile_expression(expr)?;
                             let index = self.vm.add_constant(Constant::FormatSpecifiers(
-                                format_specifiers.map(|s| s.to_string()),
+                                format_specifiers.map(|s| s.to_compact_string()),
                             ));
                             self.vm.add_op1(Op::LoadConstant, index)
                         }
@@ -283,11 +284,11 @@ impl BytecodeInterpreter {
 
         // For variables, we ignore the prefix info and only use the names
         let aliases = crate::decorator::name_and_aliases(identifier, decorators)
-            .map(|(name, _)| name.to_owned())
+            .map(|(name, _)| name.to_compact_string())
             .collect::<Vec<_>>();
         let metadata = LocalMetadata {
-            name: crate::decorator::name(decorators).map(ToOwned::to_owned),
-            url: crate::decorator::url(decorators).map(ToOwned::to_owned),
+            name: crate::decorator::name(decorators).map(CompactString::from),
+            url: crate::decorator::url(decorators).map(CompactString::from),
             description: crate::decorator::description(decorators),
             aliases: aliases.clone(),
         };
@@ -335,7 +336,7 @@ impl BytecodeInterpreter {
                 let current_depth = self.current_depth();
                 for parameter in parameters {
                     self.locals[current_depth].push(Local {
-                        identifier: parameter.1.to_string(),
+                        identifier: parameter.1.to_compact_string(),
                         depth: current_depth,
                         metadata: LocalMetadata::default(),
                     });
@@ -352,7 +353,7 @@ impl BytecodeInterpreter {
 
                 self.vm.end_function();
 
-                self.functions.insert(name.to_string(), false);
+                self.functions.insert(name.to_compact_string(), false);
             }
             Statement::DefineFunction(
                 name,
@@ -371,7 +372,7 @@ impl BytecodeInterpreter {
                 self.vm
                     .add_foreign_function(name, parameters.len()..=parameters.len());
 
-                self.functions.insert(name.to_string(), true);
+                self.functions.insert(name.to_compact_string(), true);
             }
             Statement::DefineDimension(_name, _dexprs) => {
                 // Declaring a dimension is like introducing a new type. The information
@@ -379,7 +380,7 @@ impl BytecodeInterpreter {
             }
             Statement::DefineBaseUnit(unit_name, decorators, annotation, type_) => {
                 let aliases = decorator::name_and_aliases(unit_name, decorators)
-                    .map(|(name, ap)| (name.to_owned(), ap))
+                    .map(|(name, ap)| (name.to_compact_string(), ap))
                     .collect();
 
                 self.vm
@@ -393,11 +394,11 @@ impl BytecodeInterpreter {
                                 .map(|a| a.pretty_print())
                                 .unwrap_or(type_.to_readable_type(dimension_registry, false)),
                             aliases,
-                            name: decorator::name(decorators).map(ToOwned::to_owned),
+                            name: decorator::name(decorators).map(CompactString::from),
                             canonical_name: decorator::get_canonical_unit_name(
                                 unit_name, decorators,
                             ),
-                            url: decorator::url(decorators).map(ToOwned::to_owned),
+                            url: decorator::url(decorators).map(CompactString::from),
                             description: decorator::description(decorators),
                             binary_prefixes: decorators.contains(&Decorator::BinaryPrefixes),
                             metric_prefixes: decorators.contains(&Decorator::MetricPrefixes),
@@ -406,7 +407,7 @@ impl BytecodeInterpreter {
                     .map_err(RuntimeError::UnitRegistryError)?;
 
                 let constant_idx = self.vm.add_constant(Constant::Unit(Unit::new_base(
-                    unit_name,
+                    unit_name.to_compact_string(),
                     crate::decorator::get_canonical_unit_name(unit_name, &decorators[..]),
                 )));
                 for (name, _) in decorator::name_and_aliases(unit_name, decorators) {
@@ -423,13 +424,13 @@ impl BytecodeInterpreter {
                 _readable_type,
             ) => {
                 let aliases = decorator::name_and_aliases(unit_name, decorators)
-                    .map(|(name, ap)| (name.to_owned(), ap))
+                    .map(|(name, ap)| (name.to_compact_string(), ap))
                     .collect();
 
                 let constant_idx = self.vm.add_constant(Constant::Unit(Unit::new_base(
-                    "<dummy>",
+                    CompactString::const_new("<dummy>"),
                     CanonicalName {
-                        name: "<dummy>".to_string(),
+                        name: CompactString::const_new("<dummy>"),
                         accepts_prefix: AcceptsPrefix::both(),
                     },
                 ))); // TODO: dummy is just a temp. value until the SetUnitConstant op runs
@@ -445,9 +446,9 @@ impl BytecodeInterpreter {
                             .map(|a| a.pretty_print())
                             .unwrap_or(type_.to_readable_type(dimension_registry, false)),
                         aliases,
-                        name: decorator::name(decorators).map(ToOwned::to_owned),
+                        name: decorator::name(decorators).map(CompactString::from),
                         canonical_name: decorator::get_canonical_unit_name(unit_name, decorators),
-                        url: decorator::url(decorators).map(ToOwned::to_owned),
+                        url: decorator::url(decorators).map(CompactString::from),
                         description: decorator::description(decorators),
                         binary_prefixes: decorators.contains(&Decorator::BinaryPrefixes),
                         metric_prefixes: decorators.contains(&Decorator::MetricPrefixes),
