@@ -5,6 +5,7 @@ use super::Args;
 use super::Result;
 use crate::value::Value;
 use crate::RuntimeError;
+use compact_str::CompactString;
 
 fn line_plot(mut args: Args) -> Plot {
     let mut fields = arg!(args).unsafe_as_struct_fields();
@@ -81,27 +82,27 @@ fn bar_chart(mut args: Args) -> Plot {
 }
 
 #[cfg(not(target_family = "wasm"))]
-fn show_plot(plot: Plot) -> String {
+fn show_plot(plot: Plot) -> CompactString {
     plot.show();
 
-    "Plot will be opened in the browser".into()
+    CompactString::const_new("Plot will be opened in the browser")
 }
 
 #[cfg(target_family = "wasm")]
-fn show_plot(_plot: Plot) -> String {
+fn show_plot(_plot: Plot) -> CompactString {
     // The way we could implement this would be to return plot.to_inline_html(..).
     // This would have to be retrieved on the JS side and then rendered using plotly.js.
 
-    "Plotting is currently not supported on this platform.".into()
+    CompactString::const_new("Plotting is currently not supported on this platform.")
 }
 
 pub fn show(args: Args) -> Result<Value> {
     // Dynamic dispatch hack since we don't have bounded polymorphism.
     // And no real support for generics in the FFI.
     let Value::StructInstance(info, _) = args.front().unwrap() else {
-        return Err(RuntimeError::UserError(
+        return Err(Box::new(RuntimeError::UserError(
             "Unsupported argument to 'show'.".into(),
-        ));
+        )));
     };
 
     let plot = if info.name == "LinePlot" {
@@ -109,11 +110,11 @@ pub fn show(args: Args) -> Result<Value> {
     } else if info.name == "BarChart" {
         bar_chart(args)
     } else {
-        return Err(RuntimeError::UserError(format!(
+        return Err(Box::new(RuntimeError::UserError(format!(
             "Unsupported plot type: {}",
             info.name
-        )));
+        ))));
     };
 
-    return_string!(show_plot(plot))
+    return_string!(owned = show_plot(plot))
 }
