@@ -1,3 +1,4 @@
+use compact_str::CompactString;
 use plotly::{
     color::Rgb,
     common::{Font, Line},
@@ -30,7 +31,7 @@ pub fn line_plot(xs: Vec<f64>, ys: Vec<f64>, x_label: &str, y_label: &str) -> Pl
     plot
 }
 
-pub fn bar_chart(values: Vec<f64>, x_labels: Vec<String>, value_label: &str) -> Plot {
+pub fn bar_chart(values: Vec<f64>, x_labels: Vec<CompactString>, value_label: &str) -> Plot {
     let mut plot = Plot::new();
 
     let trace = Bar::new(x_labels, values);
