@@ -1,3 +1,7 @@
+use std::sync::Arc;
+
+use compact_str::{format_compact, CompactString};
+
 use super::substitutions::{ApplySubstitution, Substitution, SubstitutionError};
 use crate::type_variable::TypeVariable;
 use crate::typed_ast::{DType, DTypeFactor, Type};
@@ -66,6 +70,14 @@ impl ConstraintSet {
         result
     }
 
+    pub(crate) fn add_equal_constraint(&mut self, lhs: &Type, rhs: &Type) -> TrivialResolution {
+        self.add(Constraint::Equal(lhs.clone(), rhs.clone()))
+    }
+
+    pub(crate) fn add_dtype_constraint(&mut self, type_: &Type) -> TrivialResolution {
+        self.add(Constraint::IsDType(type_.clone()))
+    }
+
     pub fn clear(&mut self) {
         self.constraints.clear();
     }
@@ -123,7 +135,7 @@ impl ConstraintSet {
                 remaining_constraints
                     .iter()
                     .map(|c| c.pretty_print())
-                    .collect::<Vec<String>>()
+                    .collect::<Vec<CompactString>>()
                     .join("\n"),
             ));
         }
@@ -179,7 +191,7 @@ pub enum Constraint {
     Equal(Type, Type),
     IsDType(Type),
     EqualScalar(DType),
-    HasField(Type, String, Type),
+    HasField(Type, CompactString, Type),
 }
 
 impl Constraint {
@@ -296,9 +308,9 @@ impl Constraint {
             Constraint::EqualScalar(d) if d == &DType::scalar() => Some(Satisfied::trivially()),
             Constraint::EqualScalar(dtype) => match dtype.split_first_factor() {
                 Some(((DTypeFactor::TVar(tv), k), rest)) => {
-                    let result = DType::from_factors(
+                    let result = DType::from_factors(Arc::new(
                         rest.iter().map(|(f, j)| (f.clone(), -j / k)).collect(),
-                    );
+                    ));
                     Some(Satisfied::with_substitution(Substitution::single(
                         tv.clone(),
                         Type::Dimension(result),
@@ -326,15 +338,15 @@ impl Constraint {
         }
     }
 
-    fn pretty_print(&self) -> String {
+    fn pretty_print(&self) -> CompactString {
         match self {
             Constraint::Equal(t1, t2) => {
-                format!("  {t1} ~ {t2}")
+                format_compact!("  {t1} ~ {t2}")
             }
-            Constraint::IsDType(t) => format!("  {t}: DType"),
-            Constraint::EqualScalar(d) => format!("  {d} = Scalar"),
+            Constraint::IsDType(t) => format_compact!("  {t}: DType"),
+            Constraint::EqualScalar(d) => format_compact!("  {d} = Scalar"),
             Constraint::HasField(struct_type, field_name, field_type) => {
-                format!("HasField({struct_type}, \"{field_name}\", {field_type})")
+                format_compact!("HasField({struct_type}, \"{field_name}\", {field_type})")
             }
         }
     }
