@@ -1,3 +1,6 @@
+use std::sync::Arc;
+
+use compact_str::{format_compact, CompactString, ToCompactString};
 use indexmap::IndexMap;
 use itertools::Itertools;
 
@@ -21,8 +24,8 @@ use crate::{markup as m, BaseRepresentation, BaseRepresentationFactor};
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum DTypeFactor {
     TVar(TypeVariable),
-    TPar(String),
-    BaseDimension(String),
+    TPar(CompactString),
+    BaseDimension(CompactString),
 }
 
 impl DTypeFactor {
@@ -41,7 +44,7 @@ type DtypeFactorPower = (DTypeFactor, Exponent);
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct DType {
     // Always in canonical form
-    factors: Vec<DtypeFactorPower>,
+    factors: Arc<Vec<DtypeFactorPower>>,
 }
 
 impl DType {
@@ -49,18 +52,18 @@ impl DType {
         &self.factors
     }
 
-    pub fn into_factors(self) -> Vec<DtypeFactorPower> {
+    pub fn into_factors(self) -> Arc<Vec<DtypeFactorPower>> {
         self.factors
     }
 
-    pub fn from_factors(factors: Vec<DtypeFactorPower>) -> DType {
+    pub fn from_factors(factors: Arc<Vec<DtypeFactorPower>>) -> DType {
         let mut dtype = DType { factors };
         dtype.canonicalize();
         dtype
     }
 
     pub fn scalar() -> DType {
-        DType::from_factors(vec![])
+        DType::from_factors(Arc::new(vec![]))
     }
 
     pub fn is_scalar(&self) -> bool {
@@ -75,14 +78,14 @@ impl DType {
         let mut names = vec![];
 
         if self.factors.len() == 1 && self.factors[0].1 == Exponent::from_integer(1) {
-            names.push(self.factors[0].0.name().to_string());
+            names.push(self.factors[0].0.name().to_compact_string());
         }
 
         let base_representation = self.to_base_representation();
         names.extend(registry.get_derived_entry_names_for(&base_representation));
         match &names[..] {
             [] => self.pretty_print(),
-            [single] => m::type_identifier(single.to_string()),
+            [single] => m::type_identifier(single.to_compact_string()),
             multiple => Itertools::intersperse(
                 multiple.iter().cloned().map(m::type_identifier),
                 m::dimmed(" or "),
@@ -99,11 +102,17 @@ impl DType {
     }
 
     pub fn from_type_variable(v: TypeVariable) -> DType {
-        DType::from_factors(vec![(DTypeFactor::TVar(v), Exponent::from_integer(1))])
+        DType::from_factors(Arc::new(vec![(
+            DTypeFactor::TVar(v),
+            Exponent::from_integer(1),
+        )]))
     }
 
-    pub fn from_type_parameter(name: String) -> DType {
-        DType::from_factors(vec![(DTypeFactor::TPar(name), Exponent::from_integer(1))])
+    pub fn from_type_parameter(name: CompactString) -> DType {
+        DType::from_factors(Arc::new(vec![(
+            DTypeFactor::TPar(name),
+            Exponent::from_integer(1),
+        )]))
     }
 
     pub fn deconstruct_as_single_type_variable(&self) -> Option<TypeVariable> {
@@ -116,22 +125,22 @@ impl DType {
     }
 
     pub fn from_tgen(i: usize) -> DType {
-        DType::from_factors(vec![(
+        DType::from_factors(Arc::new(vec![(
             DTypeFactor::TVar(TypeVariable::Quantified(i)),
             Exponent::from_integer(1),
-        )])
+        )]))
     }
 
     pub fn base_dimension(name: &str) -> DType {
-        DType::from_factors(vec![(
+        DType::from_factors(Arc::new(vec![(
             DTypeFactor::BaseDimension(name.into()),
             Exponent::from_integer(1),
-        )])
+        )]))
     }
 
     fn canonicalize(&mut self) {
         // Move all type-variable and tgen factors to the front, sort by name
-        self.factors.sort_by(|(f1, _), (f2, _)| match (f1, f2) {
+        Arc::make_mut(&mut self.factors).sort_by(|(f1, _), (f2, _)| match (f1, f2) {
             (DTypeFactor::TVar(v1), DTypeFactor::TVar(v2)) => v1.cmp(v2),
             (DTypeFactor::TVar(_), _) => std::cmp::Ordering::Less,
 
@@ -145,7 +154,7 @@ impl DType {
 
         // Merge powers of equal factors:
         let mut new_factors = Vec::new();
-        for (f, n) in &self.factors {
+        for (f, n) in self.factors.iter() {
             if let Some((last_f, last_n)) = new_factors.last_mut() {
                 if f == last_f {
                     *last_n += n;
@@ -154,16 +163,16 @@ impl DType {
             }
             new_factors.push((f.clone(), *n));
         }
-        self.factors = new_factors;
 
         // Remove factors with zero exponent:
-        self.factors
-            .retain(|(_, n)| *n != Exponent::from_integer(0));
+        new_factors.retain(|(_, n)| *n != Exponent::from_integer(0));
+
+        self.factors = Arc::new(new_factors);
     }
 
     pub fn multiply(&self, other: &DType) -> DType {
         let mut factors = self.factors.clone();
-        factors.extend(other.factors.clone());
+        Arc::make_mut(&mut factors).extend(other.factors.iter().cloned());
         DType::from_factors(factors)
     }
 
@@ -173,7 +182,7 @@ impl DType {
             .iter()
             .map(|(f, m)| (f.clone(), n * m))
             .collect();
-        DType::from_factors(factors)
+        DType::from_factors(Arc::new(factors))
     }
 
     pub fn inverse(&self) -> DType {
@@ -217,7 +226,7 @@ impl DType {
     fn instantiate(&self, type_variables: &[TypeVariable]) -> DType {
         let mut factors = Vec::new();
 
-        for (f, n) in &self.factors {
+        for (f, n) in self.factors.iter() {
             match f {
                 DTypeFactor::TVar(TypeVariable::Quantified(i)) => {
                     factors.push((DTypeFactor::TVar(type_variables[*i].clone()), *n));
@@ -227,12 +236,12 @@ impl DType {
                 }
             }
         }
-        Self::from_factors(factors)
+        Self::from_factors(Arc::new(factors))
     }
 
     pub fn to_base_representation(&self) -> BaseRepresentation {
         let mut factors = vec![];
-        for (f, n) in &self.factors {
+        for (f, n) in self.factors.iter() {
             match f {
                 DTypeFactor::BaseDimension(name) => {
                     factors.push(BaseRepresentationFactor(name.clone(), *n));
@@ -270,21 +279,21 @@ impl From<BaseRepresentation> for DType {
             .into_iter()
             .map(|BaseRepresentationFactor(name, exp)| (DTypeFactor::BaseDimension(name), exp))
             .collect();
-        DType::from_factors(factors)
+        DType::from_factors(Arc::new(factors))
     }
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct StructInfo {
     pub definition_span: Span,
-    pub name: String,
-    pub fields: IndexMap<String, (Span, Type)>,
+    pub name: CompactString,
+    pub fields: IndexMap<CompactString, (Span, Type)>,
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum Type {
     TVar(TypeVariable),
-    TPar(String),
+    TPar(CompactString),
     Dimension(DType),
     Boolean,
     String,
@@ -314,7 +323,7 @@ impl std::fmt::Display for Type {
                 )
             }
             Type::Struct(info) => {
-                let StructInfo { name, fields, .. } = &**info;
+                let StructInfo { name, fields, .. } = info.as_ref();
                 write!(
                     f,
                     "{name} {{{}}}",
@@ -332,7 +341,7 @@ impl std::fmt::Display for Type {
 impl PrettyPrint for Type {
     fn pretty_print(&self) -> Markup {
         match self {
-            Type::TVar(TypeVariable::Named(name)) => m::type_identifier(name.clone()),
+            Type::TVar(TypeVariable::Named(name)) => m::type_identifier(name.to_compact_string()),
             Type::TVar(TypeVariable::Quantified(_)) => {
                 unreachable!("Quantified types should not be printed")
             }
@@ -459,7 +468,7 @@ impl Type {
 
 #[derive(Debug, Clone, PartialEq)]
 pub enum StringPart<'a> {
-    Fixed(String),
+    Fixed(CompactString),
     Interpolation {
         span: Span,
         expr: Box<Expression<'a>>,
@@ -479,7 +488,7 @@ impl PrettyPrint for StringPart<'_> {
                 let mut markup = m::operator("{") + expr.pretty_print();
 
                 if let Some(format_specifiers) = format_specifiers {
-                    markup += m::text(format_specifiers.to_string());
+                    markup += m::text(format_specifiers.to_compact_string());
                 }
 
                 markup += m::operator("}");
@@ -500,7 +509,7 @@ impl PrettyPrint for &Vec<StringPart<'_>> {
 pub enum Expression<'a> {
     Scalar(Span, Number, TypeScheme),
     Identifier(Span, &'a str, TypeScheme),
-    UnitIdentifier(Span, Prefix, String, String, TypeScheme),
+    UnitIdentifier(Span, Prefix, CompactString, CompactString, TypeScheme),
     UnaryOperator(Span, UnaryOperator, Box<Expression<'a>>, TypeScheme),
     BinaryOperator(
         Option<Span>,
@@ -728,9 +737,9 @@ impl Statement<'_> {
         let mut exponents = vec![];
         self.for_all_type_schemes(&mut |type_: &mut TypeScheme| {
             if let Type::Dimension(dtype) = type_.unsafe_as_concrete() {
-                for (factor, exp) in dtype.factors {
-                    if factor == DTypeFactor::TVar(tv.clone()) {
-                        exponents.push(exp)
+                for (factor, exp) in dtype.factors.iter() {
+                    if factor == &DTypeFactor::TVar(tv.clone()) {
+                        exponents.push(*exp)
                     }
                 }
             }
@@ -858,7 +867,8 @@ fn decorator_markup(decorators: &Vec<Decorator>) -> Markup {
                         + m::operator("(")
                         + Itertools::intersperse(
                             names.iter().map(|(name, accepts_prefix, _)| {
-                                m::unit(name.to_string()) + accepts_prefix_markup(accepts_prefix)
+                                m::unit(name.to_compact_string())
+                                    + accepts_prefix_markup(accepts_prefix)
                             }),
                             m::operator(", "),
                         )
@@ -918,7 +928,7 @@ pub fn pretty_print_function_signature<'a>(
         m::operator("<")
             + Itertools::intersperse(
                 type_parameters.iter().map(|tv| {
-                    m::type_identifier(tv.unsafe_name().to_string())
+                    m::type_identifier(tv.unsafe_name().to_compact_string())
                         + if fn_type.bounds.is_dtype_bound(tv) {
                             m::operator(":") + m::space() + m::type_identifier("Dim")
                         } else {
@@ -933,7 +943,7 @@ pub fn pretty_print_function_signature<'a>(
 
     let markup_parameters = Itertools::intersperse(
         parameters.map(|(name, parameter_type)| {
-            m::identifier(name.to_string()) + m::operator(":") + m::space() + parameter_type
+            m::identifier(name.to_compact_string()) + m::operator(":") + m::space() + parameter_type
         }),
         m::operator(", "),
     )
@@ -944,7 +954,7 @@ pub fn pretty_print_function_signature<'a>(
 
     m::keyword("fn")
         + m::space()
-        + m::identifier(function_name.to_string())
+        + m::identifier(function_name.to_compact_string())
         + markup_type_parameters
         + m::operator("(")
         + markup_parameters
@@ -965,7 +975,7 @@ impl PrettyPrint for Statement<'_> {
             )) => {
                 m::keyword("let")
                     + m::space()
-                    + m::identifier(identifier.to_string())
+                    + m::identifier(identifier.to_compact_string())
                     + m::operator(":")
                     + m::space()
                     + readable_type.clone()
@@ -1011,7 +1021,7 @@ impl PrettyPrint for Statement<'_> {
                         plv += m::nl()
                             + introducer_keyword
                             + m::space()
-                            + m::identifier(identifier.to_string())
+                            + m::identifier(identifier.to_compact_string())
                             + m::operator(":")
                             + m::space()
                             + readable_type.clone()
@@ -1039,12 +1049,14 @@ impl PrettyPrint for Statement<'_> {
             }
             Statement::Expression(expr) => expr.pretty_print(),
             Statement::DefineDimension(identifier, dexprs) if dexprs.is_empty() => {
-                m::keyword("dimension") + m::space() + m::type_identifier(identifier.to_string())
+                m::keyword("dimension")
+                    + m::space()
+                    + m::type_identifier(identifier.to_compact_string())
             }
             Statement::DefineDimension(identifier, dexprs) => {
                 m::keyword("dimension")
                     + m::space()
-                    + m::type_identifier(identifier.to_string())
+                    + m::type_identifier(identifier.to_compact_string())
                     + m::space()
                     + m::operator("=")
                     + m::space()
@@ -1058,7 +1070,7 @@ impl PrettyPrint for Statement<'_> {
                 decorator_markup(decorators)
                     + m::keyword("unit")
                     + m::space()
-                    + m::unit(identifier.to_string())
+                    + m::unit(identifier.to_compact_string())
                     + m::operator(":")
                     + m::space()
                     + annotation
@@ -1077,7 +1089,7 @@ impl PrettyPrint for Statement<'_> {
                 decorator_markup(decorators)
                     + m::keyword("unit")
                     + m::space()
-                    + m::unit(identifier.to_string())
+                    + m::unit(identifier.to_compact_string())
                     + m::operator(":")
                     + m::space()
                     + readable_type.clone()
@@ -1181,11 +1193,11 @@ fn pretty_print_binop(op: &BinaryOperator, lhs: &Expression, rhs: &Expression) -
                 // Fuse multiplication of a scalar and a unit to a quantity
                 pretty_scalar(*s)
                     + m::space()
-                    + m::unit(format!("{}{}", prefix.as_string_long(), full_name))
+                    + m::unit(format_compact!("{}{}", prefix.as_string_long(), full_name))
             }
             (Expression::Scalar(_, s, _), Expression::Identifier(_, name, _type)) => {
                 // Fuse multiplication of a scalar and identifier
-                pretty_scalar(*s) + m::space() + m::identifier(name.to_string())
+                pretty_scalar(*s) + m::space() + m::identifier(name.to_compact_string())
             }
             _ => {
                 let add_parens_if_needed = |expr: &Expression| {
@@ -1275,9 +1287,9 @@ impl PrettyPrint for Expression<'_> {
 
         match self {
             Scalar(_, n, _) => pretty_scalar(*n),
-            Identifier(_, name, _type) => m::identifier(name.to_string()),
+            Identifier(_, name, _type) => m::identifier(name.to_compact_string()),
             UnitIdentifier(_, prefix, _name, full_name, _type) => {
-                m::unit(format!("{}{}", prefix.as_string_long(), full_name))
+                m::unit(format_compact!("{}{}", prefix.as_string_long(), full_name))
             }
             UnaryOperator(_, self::UnaryOperator::Negate, expr, _type) => {
                 m::operator("-") + with_parens(expr)
@@ -1291,7 +1303,7 @@ impl PrettyPrint for Expression<'_> {
             BinaryOperator(_, op, lhs, rhs, _type) => pretty_print_binop(op, lhs, rhs),
             BinaryOperatorForDate(_, op, lhs, rhs, _type) => pretty_print_binop(op, lhs, rhs),
             FunctionCall(_, _, name, args, _type) => {
-                m::identifier(name.to_string())
+                m::identifier(name.to_compact_string())
                     + m::operator("(")
                     + itertools::Itertools::intersperse(
                         args.iter().map(|e| e.pretty_print()),
@@ -1335,7 +1347,7 @@ impl PrettyPrint for Expression<'_> {
                         m::space()
                             + itertools::Itertools::intersperse(
                                 exprs.iter().map(|(n, e)| {
-                                    m::identifier(n.to_string())
+                                    m::identifier(n.to_compact_string())
                                         + m::operator(":")
                                         + m::space()
                                         + e.pretty_print()
@@ -1348,7 +1360,7 @@ impl PrettyPrint for Expression<'_> {
                     + m::operator("}")
             }
             AccessField(_, _, expr, attr, _, _) => {
-                expr.pretty_print() + m::operator(".") + m::identifier(attr.to_string())
+                expr.pretty_print() + m::operator(".") + m::identifier(attr.to_compact_string())
             }
             List(_, elements, _) => {
                 m::operator("[")
@@ -1444,7 +1456,7 @@ mod tests {
             .clone()
     }
 
-    fn pretty_print(stmt: &Statement) -> String {
+    fn pretty_print(stmt: &Statement) -> CompactString {
         let markup = stmt.pretty_print();
 
         (PlainTextFormatter {}).format(&markup, false)
